package com.fr.plugin.xxxxx.factory.loader;

import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xxxxx.factory.util.DataUtil;
import com.fr.plugin.xxxxx.factory.util.HttpUtil;
import com.fr.plugin.xxxxx.factory.util.TokenFactory;
import com.fr.script.Calculator;
import com.fr.stable.ParameterProvider;
import com.fr.stable.StringUtils;
import com.fr.third.org.apache.commons.lang3.time.StopWatch;
import com.tptj.plugin.hg.impl.AbstractLoader;

import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author fr.open
 * @Date 2020/12/4
 * @Description
 **/
public class PageDataFactoryLoader extends AbstractLoader {

    @Override
    public Object load(Calculator calculator, ParameterProvider[] parameterProviders, String others) {
        StopWatch stopWatch = StopWatch.createStarted();
        JSONObject param =  new JSONObject(others);//DataUtil.getJSON(others);
        if(param.isEmpty()){
            return null;
        }
        if (!param.has("method") || StringUtils.isBlank(param.getString("method"))) {
            ;
            param.put("method", DataUtil.getProp("method"));
        }
        if (!param.has("tokenUrl") || StringUtils.isBlank(param.getString("tokenUrl"))) {
            param.put("tokenUrl", DataUtil.getProp("tokenUrl"));
        } else {
            if (!param.getString("tokenUrl").startsWith("http")) {
                param.put("tokenUrl", DataUtil.getProp("prefix") + param.getString("tokenUrl"));
            }
        }
        if (!param.has("userid") || StringUtils.isBlank(param.getString("userid"))) {
            param.put("userid", DataUtil.getProp("userid"));
        }
        if (!param.has("value") || StringUtils.isBlank(param.getString("value"))) {
            param.put("value", DataUtil.getProp("value"));
        }
        if (!param.has("url") || StringUtils.isBlank(param.getString("url"))) {
            param.put("url", DataUtil.getProp("url"));
        } else {
            if (!param.getString("url").startsWith("http")) {
                param.put("url", DataUtil.getProp("prefix") + param.getString("url"));
            }
        }
        String res = StringUtils.EMPTY;
        String token = TokenFactory.getToken(param.getString("tokenUrl"), param.getString("userid"), param.getString("value"));
        if (StringUtils.isBlank(token)) {
            FineLoggerFactory.getLogger().error("get token is null");
            return null;
        }
        FineLoggerFactory.getLogger().info("get token is {} by url {}", token, param.getString("tokenUrl"));
        Map<String, Object> headerMap = param.getJSONObject("header") == null ? null : DataUtil.parseHeader(param.getJSONObject("header"));
        headerMap.put("openid", token);
        FineLoggerFactory.getLogger().info("header is {}", headerMap);
        Map<String, Object> paraMap = param.getJSONObject("para") == null ? null : DataUtil.parseHeader(param.getJSONObject("para"));
        FineLoggerFactory.getLogger().info("para is {}", paraMap);
        JSONObject body = param.getJSONObject("data") == null ? new JSONObject() : param.getJSONObject("data");
        FineLoggerFactory.getLogger().info("data is {}", body);
        if (param.getString("method").equalsIgnoreCase("GET")) {
            res = HttpUtil.sendGet(param.getString("url"), paraMap, headerMap, null);
        } else if (param.getString("method").equalsIgnoreCase("POST")) {
            res = HttpUtil.doJSONPost(param.getString("url"), headerMap, body, paraMap, null);
        }
        JSONObject data = JSONObject.create();
        //获取分页
        if (StringUtils.isNotBlank(res)) {
            JSONObject object = new JSONObject(res);
            data = object;
            while (object.has("hasNextPage") && object.getBoolean("hasNextPage")) {
                int page = object.getInt("currentPage");
                FineLoggerFactory.getLogger().info("the url {} current page is {} has next page", param.getString("url"), ++page);
                if (param.getString("method").equalsIgnoreCase("GET")) {
                    JSONObject p = param.getJSONObject("header").getJSONObject("params");
                    p = p == null ? JSONObject.create() : p;
                    p.put("pageIndex", page);
                    headerMap.put("params", p.toString());
                    res = HttpUtil.sendGet(param.getString("url"), paraMap, headerMap, null);
                } else if (param.getString("method").equalsIgnoreCase("POST")) {
                    body.put("pageIndex", page);
                    res = HttpUtil.doJSONPost(param.getString("url"), headerMap, body, paraMap, null);
                }
                JSONObject temp = new JSONObject(res);
                data.getJsonArray("data").addAll(temp.getJSONArray("data"));
                object = temp;
            }
        }
        FineLoggerFactory.getLogger().info("the url {} response data is {}", param.getString("url"), data);
        stopWatch.stop();
        FineLoggerFactory.getLogger().error("load page openApi data cost {}",stopWatch.getTime(TimeUnit.SECONDS));
        return data;
    }


    @Override
    public String getName() {
        return "openApi分页";
    }

    @Override
    public String getDefaultConfig() {
        return "{\n" +
                "    \"url\": \"\",\n" +
                "    \"data\": {\n" +
                "    },\n" +
                "    \"para\": {},\n" +
                "    \"header\": {},\n" +
                "    \"method\": \"POST\",\n" +
                "    \"tokenUrl\": \"https://xxxxx/sm/session\",\n" +
                "    \"userid\": \"\",\n" +
                "    \"value\": \"\"\n" +
                "}";
    }
}
