package com.fr.plugin.xxxxx.factory.resolver;

import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.xxxxx.factory.util.DataUtil;
import com.fr.stable.StringUtils;
import com.tptj.plugin.hg.impl.AbstractResolver;
import com.tptj.plugin.hg.stable.SimpleDataModel;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author fr.open
 * @Date 2021/1/27
 * @Description
 **/
public class GroupResolver extends AbstractResolver {
    @Override
    public SimpleDataModel parse(Object data, String others) {
        SimpleDataModel dataModel = new SimpleDataModel();
        List<Object[]> datas = new ArrayList<>();
        List<String> columns = new ArrayList<>();
        JSONObject config = new JSONObject(others);
        if (!config.has("group") || StringUtils.isBlank(config.getString("group"))) {
            return null;
        }
        if (!config.has("data") || StringUtils.isBlank(config.getString("data"))) {
            return null;
        }
        String configGroup = config.getString("group");
        int index = configGroup.indexOf("==");
        String groupName = configGroup;
        String groupData = null;
        if(index != -1){
            groupName = configGroup.substring(0,index);
            groupData = configGroup.substring(index+2);
        }
        String configData = config.getString("data");
        columns.add(groupName);
        columns.add(configData);
        JSONObject jsonData = new JSONObject(data.toString());
        if (!jsonData.has("data")) {
            return null;
        }
        JSONArray array = jsonData.getJSONArray("data");
        Map<String, Set<Object>> map = new HashMap();
        for (int i = 0; i < array.size(); i++) {
            JSONObject object = array.getJSONObject(i);
            if (!object.has(groupName) || StringUtils.isBlank(object.getString(groupName))) {
                continue;
            }
            if (!object.has(configData) || StringUtils.isBlank(object.getString(configData))) {
                continue;
            }
            String group = object.getString(groupName);
            if(StringUtils.isNotBlank(groupData) && !hasGroup(group,groupData)){
                continue;
            }
            String d = object.getString(configData);
            Set<Object> list = new HashSet<>();
            if (map.containsKey(group)) {
                list = map.get(group);
            }
            list.add(d);
            map.put(group, list);
        }
        Iterator<String> iterator = map.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Set<Object> value = map.get(key);
            Object[] row = new Object[]{key, value.stream().map(e -> e.toString()).collect(Collectors.joining(","))};
            datas.add(row);
        }
        dataModel.setCols(DataUtil.translate(columns,config.getBoolean("translate")));
        dataModel.setDatas(datas);
        return dataModel;
    }

    private boolean hasGroup(String group, String config) {
        String[] arr = config.split(",");
        for (String str : arr){
            if(StringUtils.equals(str,group)){
                return true;
            }
        }
        return false;
    }

    @Override
    public String getName() {
        return "分组解析器";
    }

    @Override
    public String getDefaultConfig() {
        return "{\n" +
                "    \"group\":\"\",\n" +
                "    \"data\":\"\",\n" +
                "    \"translate\":\"false\",\n" +
                "}";
    }
}
