package com.fr.plugin.xxxxx.factory.resolver;

import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xxxxx.factory.util.DataUtil;
import com.fr.stable.StringUtils;
import com.jayway.jsonpath.JsonPath;
import com.tptj.plugin.hg.impl.AbstractResolver;
import com.tptj.plugin.hg.stable.SimpleDataModel;

import java.util.*;

/**
 * @Author fr.open
 * @Date 2020/12/7
 * @Description
 **/
public class OpenApiFind extends AbstractResolver {
    @Override
    public SimpleDataModel parse(Object data, String others) {
        SimpleDataModel dataModel = new SimpleDataModel();
        List<Object[]> datas = new ArrayList<>();
        List<String> columns = new ArrayList<>();
        JSONObject param = new JSONObject(others);
        String id = StringUtils.EMPTY;
        if (!param.has("data") || StringUtils.isBlank(param.getString("data"))) {
            FineLoggerFactory.getLogger().error("data is not null");
            return null;
        }
        if (param.has("id") && StringUtils.isNotBlank(param.getString("id"))) {
            id = param.getString("id");
        }
        List res = JsonPath.read(data.toString(), param.getString("data"));

        if (res instanceof List) {
            List<Map> list = (List<Map>) res;
            if (columns.isEmpty()) {
                for (Map map : list) {
                    if (map.keySet().size() > columns.size()) {
                        columns = new ArrayList(map.keySet());
                    }
                }
            }
            for (Map map : list) {
                Object[] row = new Object[columns.size()];
                Iterator<String> it = columns.iterator();
                int i = 0;
                while (it.hasNext()) {
                    row[i] = map.get(it.next());
                    i++;
                }
                datas.add(row);
            }

        } else if (res instanceof Map) {
            Map map = (Map) res;
            if (columns.isEmpty()) {
                columns = new ArrayList(map.keySet());
            }
            Object[] row = new Object[columns.size()];
            Iterator<String> it = columns.iterator();
            int i = 0;
            while (it.hasNext()) {
                row[i] = map.get(it.next());
                i++;
            }
            datas.add(row);
        } else {
            if (columns.isEmpty()) {
                columns.add("data");
            }
            datas.add(new Object[]{res});
        }
        dataModel.setCols(DataUtil.translate(columns, param.getBoolean("translate")));

        if (StringUtils.isBlank(id) || StringUtils.isBlank(param.getString("name")) || StringUtils.isBlank("parent")) {
            dataModel.setDatas(datas);
        } else {
            if (columns.indexOf(param.getString("name")) == -1 || columns.indexOf(param.getString("parent")) == -1) {
                dataModel.setDatas(datas);
            } else {
                List<Object[]> find = new ArrayList<>();
                int nameIndex = columns.indexOf(param.getString("name"));
                int parentIndex = columns.indexOf(param.getString("parent"));
                //生成map方便检索数据
                Map<String, Object[]> map = new HashMap<>();
                for (int i = 0; i < datas.size(); i++) {
                    Object[] row = datas.get(i);
                    map.put(row[nameIndex].toString(), row);
                }
                //String tmp = id;
                Set<String> ids = new HashSet<>();
                for (int i = 0; i < id.split(",").length; i++) {
                    String tmp = id.split(",")[i];
                    while (map.containsKey(tmp)) {
                        if(!ids.contains(tmp)){
                            Object[] row = map.get(tmp);
                            find.add(row);
                            ids.add(tmp);
                            tmp = row[parentIndex] == null?"":row[parentIndex].toString();
                        }else {
                            tmp = StringUtils.EMPTY;
                        }
                    }
                }

                dataModel.setDatas(find);
            }
        }
        return DataUtil.sort(dataModel,param.getString("sort"));
    }


    @Override
    public String getName() {
        return "OpenApi树查找器";
    }

    @Override
    public String getDefaultConfig() {
        return "{\n" +
                "    \"data\":\"$.data\",\n" +
                "    \"id\":\"\",\n" +
                "    \"name\":\"dn\",\n" +
                "    \"parent\":\"parentDn\",\n" +
                "    \"sort\":\"\",\n" +
                "    \"translate\":\"false\",\n" +
                "}";
    }
}
