package com.fr.plugin.xxxxx.factory.resolver;

import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xxxxx.factory.util.DataUtil;
import com.fr.stable.StringUtils;
import com.jayway.jsonpath.JsonPath;
import com.tptj.plugin.hg.impl.AbstractResolver;
import com.tptj.plugin.hg.stable.SimpleDataModel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @Author fr.open
 * @Date 2020/12/7
 * @Description
 **/
public class OpenApiResolver extends AbstractResolver {
    @Override
    public SimpleDataModel parse(Object data, String others) {
        SimpleDataModel dataModel = new SimpleDataModel();
        List<Object[]> datas = new ArrayList<>();
        List<String> columns = new ArrayList<>();
        JSONObject param = new JSONObject(others);
        if (!param.has("data") || StringUtils.isBlank(param.getString("data"))) {
            FineLoggerFactory.getLogger().error("data is not null");
            return null;
        }
        if (param.has("column") && StringUtils.isNotBlank(param.getString("column"))) {
            Map map = JsonPath.read(data.toString(), param.getString("column"));
            Iterator<String> it = map.keySet().iterator();
            while (it.hasNext()) {
                columns.add(String.valueOf(map.get(it.next())));
            }
        }
        List res = JsonPath.read(data.toString(), param.getString("data"));

        if (res instanceof List) {
            List<Map> list = (List<Map>) res;
            if (columns.isEmpty()) {
                for (Map map : list) {
                    if (map.keySet().size() > columns.size()) {
                        columns = new ArrayList(map.keySet());
                    }
                }
            }
            for (Map map : list) {
                Object[] row = new Object[columns.size()];
                Iterator<String> it = map.keySet().iterator();
                int i = 0;
                while (it.hasNext()) {
                    row[i] = map.get(it.next());
                    i++;
                }
                datas.add(row);
            }

        } else if (res instanceof Map) {
            Map map = (Map) res;
            if (columns.isEmpty()) {
                columns = new ArrayList(map.keySet());
            }
            Object[] row = new Object[columns.size()];
            Iterator<String> it = map.keySet().iterator();
            int i = 0;
            while (it.hasNext()) {
                row[i] = map.get(it.next());
                i++;
            }
            datas.add(row);
        } else {
            if (columns.isEmpty()) {
                columns.add("data");
            }
            datas.add(new Object[]{res});
        }
        dataModel.setCols(DataUtil.translate(columns, param.getBoolean("translate")));
        dataModel.setDatas(datas);
        return DataUtil.sort(dataModel,param.getString("sort"));
    }


    @Override
    public String getName() {
        return "OpenApi解析器";
    }

    @Override
    public String getDefaultConfig() {
        return "{\n" +
                "    \"column\":\"$[0]\",\n" +
                "    \"data\":\"$[1:]\",\n" +
                "    \"translate\":\"false\",\n" +
                "}";
    }
}
