package com.fr.plugin.xxxxx.factory.util;

import com.fr.general.PropertiesUtils;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xxxxx.factory.EncodeDBAccessProvider;
import com.fr.plugin.xxxxx.factory.dao.EncodeDao;
import com.fr.plugin.xxxxx.factory.entity.XxxxxEncodeEntity;
import com.fr.stable.StringUtils;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.tptj.plugin.hg.stable.SimpleDataModel;

import java.util.*;

/**
 * @Author fr.open
 * @Date 2020/12/27
 * @Description
 **/
public class DataUtil {

    public static Map<String, Object> parseHeader(JSONObject object) {
        Map<String, Object> map = new HashMap<>(16);
        Iterator<String> it = object.keys();
        while (it.hasNext()) {
            String key = it.next();
            if (object.get(key) != null) {
                map.put(key, object.get(key).toString());
            }
        }
        return map;
    }

    public static JSONObject getJSON(String param) {
        try {
            FineLoggerFactory.getLogger().info("parse json string is {}", param);
            return new JSONObject(param);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("json配置异常");
        }
        return JSONObject.EMPTY;
    }

    public static SimpleDataModel sort(SimpleDataModel dataModel, String sort) {
        final Integer[] index = {dataModel.getCols().indexOf(sort)};
        if (index[0] == -1) {
            return dataModel;
        }
        Collections.sort(dataModel.getDatas(), new Comparator<Object[]>() {
            @Override
            public int compare(Object[] o1, Object[] o2) {
                char[] chars1 = (o1[index[0]] == null ? "" : String.valueOf(o1[index[0]])).toCharArray();
                char[] chars2 = (o2[index[0]] == null ? "" : String.valueOf(o2[index[0]])).toCharArray();
                int i = 0;
                while (i < chars1.length && i < chars2.length) {
                    if (chars1[i] > chars2[i]) {
                        return 1;
                    } else if (chars1[i] < chars2[i]) {
                        return -1;
                    } else {
                        i++;
                    }
                }
                if (i == chars1.length) {  //o1到头
                    return -1;
                }
                if (i == chars2.length) { //o2到头
                    return 1;
                }
                return 0;
            }
        });
        return dataModel;
    }

    public static String getProp(String key) {
        try {
            XxxxxEncodeEntity encodeEntity = EncodeDBAccessProvider.getDbAccessor().runDMLAction(new DBAction<XxxxxEncodeEntity>() {

                @Override
                public XxxxxEncodeEntity run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(EncodeDao.class).findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("prop", key)));
                }
            });
            if (encodeEntity != null && StringUtils.isNotBlank(encodeEntity.getEncode())) {
                return GCMUtils.decode(encodeEntity.getEncode());
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return PropertiesUtils.getProperties("conf").getProperty(key);
    }

    public static List<String> translate(List<String> columns, Boolean translate) {
        if(translate == null || !translate){
            return columns;
        }
        byte[] bytes = ResourceIOUtils.readBytes("/resources/transfer.json");
        if (bytes == null) {
            return columns;
        }
        try {
            JSONObject object = new JSONObject(new String(bytes));
            for (int i = 0; i < columns.size(); i++) {
                String col = columns.get(i);
                if (object.has(col) && StringUtils.isNotBlank(object.getString(col))) {
                    columns.set(i, object.getString(col));
                }
            }

        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return columns;
    }
}
