package com.fr.plugin.xxxxx.factory.util;

import com.fr.general.PropertiesUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.stable.StringUtils;
import com.fr.third.org.apache.commons.codec.binary.Base64;

import javax.crypto.*;
import javax.crypto.spec.GCMParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

/**
 * @author ezreal
 * @version 10.0
 * @date 2021/3/22
 */
public class GCMUtils {

    /*public static void main(String[] args) {
        System.out.println(UUID.randomUUID().toString().replaceAll("-",""));
        System.out.println(encode("fanruan@123","RoKH9Yd101zEfXIz8poYrw=="));
    }*/

    public static String encode(String txt, String keytext) {
        try {
            byte[] bytes = Base64.decodeBase64(keytext);
            SecretKey key = new SecretKeySpec(bytes, "AES");
            Cipher cipher = Cipher.getInstance("AES/GCM/PKCS5Padding");
            cipher.init(Cipher.ENCRYPT_MODE, key);
            byte[] iv = cipher.getIV();
            assert iv.length == 12;
            byte[] encryptData = cipher.doFinal(txt.getBytes());
            assert encryptData.length == txt.getBytes().length + 16;
            byte[] message = new byte[12 + txt.getBytes().length + 16];
            System.arraycopy(iv, 0, message, 0, 12);
            System.arraycopy(encryptData, 0, message, 12, encryptData.length);
            return Base64.encodeBase64String(message);
        } catch (NoSuchAlgorithmException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (NoSuchPaddingException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (InvalidKeyException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (IllegalBlockSizeException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (BadPaddingException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        }
        return StringUtils.EMPTY;
    }

    public static String decode(String txt, String keytext) {
        try {
            byte[] bytes = Base64.decodeBase64(keytext);
            SecretKey key = new SecretKeySpec(bytes, "AES");
            Cipher cipher = Cipher.getInstance("AES/GCM/PKCS5Padding");
            byte[] message = Base64.decodeBase64(txt);
            if (message.length < 12 + 16) throw new IllegalArgumentException();
            GCMParameterSpec params = new GCMParameterSpec(128, message, 0, 12);
            cipher.init(Cipher.DECRYPT_MODE, key, params);
            byte[] decryptData = cipher.doFinal(message, 12, message.length - 12);
            return new String(decryptData);
        } catch (NoSuchAlgorithmException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (NoSuchPaddingException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (InvalidKeyException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (InvalidAlgorithmParameterException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (IllegalBlockSizeException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        } catch (BadPaddingException e) {
            FineLoggerFactory.getLogger().error(e.getMessage());
        }
        return StringUtils.EMPTY;
    }

    public static String decode(String encode) {
        String key = PropertiesUtils.getProperties("conf").getProperty("keytext");
        return decode(encode,key);
    }

    public static String encode(String key) {
        String keytext = PropertiesUtils.getProperties("conf").getProperty("keytext");
        return encode(key,keytext);
    }
}

