package com.fr.plugin.xxxxx.factory.util;

import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.xxxxx.factory.entity.Token;
import com.fr.stable.StringUtils;

import java.time.Instant;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author fr.open
 * @Date 2021/2/2
 * @Description
 **/
public class TokenFactory {

    private static Token token;

    private static TokenFactory instance;

    private TokenFactory() {

    }

    public static TokenFactory getInstance() {
        if (instance == null) {
            instance = new TokenFactory();
        }
        return instance;
    }

    public static String getToken(String tokenUrl, String userid, String value) {
        String token = getTempToken();
        if (StringUtils.isNotBlank(token)) {
            return token;
        }
        return refreshToken(tokenUrl, userid, value);

    }

    private static String getTempToken() {
        if (token == null) {
            return null;
        }
        if (token.getTimeout() < System.currentTimeMillis()) {
            return null;
        }
        return token.getToken();
    }

    private static String refreshToken(String tokenUrl, String userid, String value) {
        JSONObject object = JSONObject.create().put("userid", userid).put("value", value);
        /*FineLoggerFactory.getLogger().info("get token param is {}", object);*/
        Map<String, Object> header = new HashMap<>();
        header.put("Accept", "application/json");
        header.put("Content-Type", "application/json;charset=UTF-8");
        String res = HttpUtil.doJSONPut(tokenUrl, header, object, null, null);
        FineLoggerFactory.getLogger().info("get token res is {} by url {}", res, tokenUrl);
        JSONObject resObj = new JSONObject(res);
        if (resObj.containsKey("data") && StringUtils.isNotBlank("data")) {
            token = new Token(resObj.getString("data"), Instant.now().plusSeconds(28 * 60).toEpochMilli());
            return resObj.getString("data");
        }
        return StringUtils.EMPTY;
    }
}
